@@ -8,7 +8,7 @@ import textwrap
 
 import numpy as np
 import matplotlib.pyplot as plt
-from matplotlib import patches, cm, colors
+from matplotlib import cm
 
 import astropy.wcs
 import astropy.units as u
@@ -19,13 +19,12 @@ from astropy.coordinates import SkyCoord, UnitSphericalRepresentation
 import sunpy.io as io
 # The next two are not used but are called to register functions with external modules
 import sunpy.coordinates
-import sunpy.cm
+import sunpy.visualization.colormaps
 from sunpy import config
 from sunpy.visualization import wcsaxes_compat, axis_labels_from_ctype, peek_show
 from sunpy.sun import constants
 from sunpy.coordinates import sun
 from sunpy.time import parse_time, is_time
-from sunpy.image.transform import affine_transform
 from sunpy.image.resample import reshape_image_to_4d_superpixel
 from sunpy.image.resample import resample as sunpy_image_resample
 from sunpy.coordinates import get_earth
@@ -199,10 +198,12 @@ class GenericMap(NDData):
         if self.dtype == np.uint8:
             norm = None
         else:
+            # Put import here to reduce sunpy.map import time
+            from matplotlib import colors
             norm = colors.Normalize()
 
         # Visualization attributes
-        self.plot_settings = {'cmap': cm.gray,
+        self.plot_settings = {'cmap': 'gray',
                               'norm': norm,
                               'interpolation': 'nearest',
                               'origin': 'lower'
@@ -228,7 +229,7 @@ class GenericMap(NDData):
                    Observation Date:\t {date}
                    Exposure Time:\t\t {dt:f}
                    Dimension:\t\t {dim}
-                   Coordinate System:\t {coord.name}
+                   Coordinate System:\t {coord}
                    Scale:\t\t\t {scale}
                    Reference Pixel:\t {refpix}
                    Reference Coord:\t {refcoord}
@@ -238,10 +239,10 @@ class GenericMap(NDData):
                                dt=self.exposure_time,
                                dim=u.Quantity(self.dimensions),
                                scale=u.Quantity(self.scale),
-                               coord=self.coordinate_frame,
+                               coord=self._coordinate_frame_name,
                                refpix=u.Quantity(self.reference_pixel),
-                               refcoord=u.Quantity((self.reference_coordinate.data.lon,
-                                                    self.reference_coordinate.data.lat)),
+                               refcoord=u.Quantity((self._reference_longitude,
+                                                    self._reference_latitude)),
                                tmf=TIME_FORMAT) + self.data.__repr__()
 
     @classmethod
@@ -265,7 +266,28 @@ class GenericMap(NDData):
         """
         The `~astropy.wcs.WCS` property of the map.
         """
-        w2 = astropy.wcs.WCS(naxis=2)
+        # Construct the WCS based on the FITS header, but don't "do_set" which
+        # analyses the FITS header for correctness.
+        with warnings.catch_warnings():
+            # Ignore warnings we may raise when constructing the fits header about dropped keys.
+            warnings.simplefilter("ignore", SunpyUserWarning)
+            try:
+                w2 = astropy.wcs.WCS(header=self.fits_header, _do_set=False)
+            except Exception:
+                warnings.warn("Unable to treat `.meta` as a FITS header, assuming a simple WCS.")
+                w2 = astropy.wcs.WCS(naxis=2)
+
+        # If the FITS header is > 2D pick the first 2 and move on.
+        # This will require the FITS header to be valid.
+        if w2.naxis > 2:
+            # We have to change this or else the WCS doesn't parse properly, even
+            # though we don't care about the third dimension. This applies to both
+            # EIT and IRIS data, it is here to reduce the chances of silly errors.
+            if self.meta.get('cdelt3', None) == 0:
+                w2.wcs.cdelt[2] = 1e-10
+
+            w2 = w2.sub([1, 2])
+
         w2.wcs.crpix = u.Quantity(self.reference_pixel)
         # Make these a quantity array to prevent the numpy setting element of
         # array with sequence error.
@@ -294,15 +316,28 @@ class GenericMap(NDData):
         if {'HPLN', 'HPLT'} <= ctypes or {'SOLX', 'SOLY'} <= ctypes:
             w2.heliographic_observer = self.observer_coordinate
 
+        # Validate the WCS here.
+        w2.wcs.set()
         return w2
 
     @property
     def coordinate_frame(self):
         """
         An `astropy.coordinates.BaseFrame` instance created from the coordinate
-        information for this Map.
+        information for this Map, or None if the frame cannot be determined.
         """
-        return astropy.wcs.utils.wcs_to_celestial_frame(self.wcs)
+        try:
+            return astropy.wcs.utils.wcs_to_celestial_frame(self.wcs)
+        except ValueError as e:
+            warnings.warn(f'Could not determine coordinate frame from map metadata',
+                          SunpyUserWarning)
+            return None
+
+    @property
+    def _coordinate_frame_name(self):
+        if self.coordinate_frame is None:
+            return 'Unknown'
+        return self.coordinate_frame.name
 
     def _as_mpl_axes(self):
         """
@@ -899,9 +934,8 @@ class GenericMap(NDData):
 
     @u.quantity_input
     def resample(self, dimensions: u.pixel, method='linear'):
-        """Returns a new Map that has been resampled up or down
-
-        Arbitrary resampling of the Map to new dimension sizes.
+        """
+        Resample to new dimension sizes.
 
         Uses the same parameters and creates the same co-ordinate lookup points
         as IDL''s congrid routine, which apparently originally came from a
@@ -910,24 +944,25 @@ class GenericMap(NDData):
         Parameters
         ----------
         dimensions : `~astropy.units.Quantity`
-            Pixel dimensions that new Map should have.
-            Note: the first argument corresponds to the 'x' axis and the second
-            argument corresponds to the 'y' axis.
-        method : {'neighbor' | 'nearest' | 'linear' | 'spline'}
+            Output pixel dimensions. The first argument corresponds to the 'x'
+            axis and the second argument corresponds to the 'y' axis.
+        method : str
             Method to use for resampling interpolation.
-                * neighbor - Closest value from original data
-                * nearest and linear - Uses n x 1-D interpolations using
-                  scipy.interpolate.interp1d
-                * spline - Uses ndimage.map_coordinates
+                * ``'neighbor'`` - Take closest value from original data.
+                * ``'nearest'`` and ``'linear'`` - Use n x 1-D interpolations using
+                  `scipy.interpolate.interp1d`.
+                * ``'spline'`` - Uses piecewise polynomials (splines) for mapping the input
+                  array to new coordinates by interpolation using
+                  `ndimage.map_coordinates`.
 
         Returns
         -------
         out : `~sunpy.map.GenericMap` or subclass
-            A new Map which has been resampled to the desired dimensions.
+            Resampled map
 
         References
         ----------
-        * `Rebinning <https://scipy-cookbook.readthedocs.io/items/Rebinning.html>`_
+        `Rebinning <https://scipy-cookbook.readthedocs.io/items/Rebinning.html>`_
         """
 
         # Note: because the underlying ndarray is transposed in sense when
@@ -965,7 +1000,8 @@ class GenericMap(NDData):
         new_map = self._new_instance(new_data, new_meta, self.plot_settings)
         return new_map
 
-    def rotate(self, angle=None, rmatrix=None, order=4, scale=1.0,
+    @u.quantity_input
+    def rotate(self, angle: u.deg = None, rmatrix=None, order=4, scale=1.0,
                recenter=False, missing=0.0, use_scipy=False):
         """
         Returns a new rotated and rescaled map.
@@ -1029,36 +1065,16 @@ class GenericMap(NDData):
         transformations, situations when the underlying data is modified prior
         to rotation, and differences from IDL's rot().
         """
+        # Put the import here to reduce sunpy.map import time
+        from sunpy.image.transform import affine_transform
+
         if angle is not None and rmatrix is not None:
-            raise ValueError("You cannot specify both an angle and a matrix")
+            raise ValueError("You cannot specify both an angle and a rotation matrix.")
         elif angle is None and rmatrix is None:
             rmatrix = self.rotation_matrix
 
-        # This is out of the quantity_input decorator. To allow the angle=None
-        # case. See https://github.com/astropy/astropy/issues/3734
-        if angle:
-            try:
-                equivalent = angle.unit.is_equivalent(u.deg)
-
-                if not equivalent:
-                    raise u.UnitsError("Argument '{}' to function '{}'"
-                                       " must be in units convertable to"
-                                       " '{}'.".format('angle', 'rotate',
-                                                        u.deg.to_string()))
-
-            # Either there is no .unit or no .is_equivalent
-            except AttributeError:
-                if hasattr(angle, "unit"):
-                    error_msg = "a 'unit' attribute without an 'is_equivalent' method"
-                else:
-                    error_msg = "no 'unit' attribute"
-                raise TypeError("Argument '{}' to function '{}' has {}. "
-                                "You may want to pass in an astropy Quantity instead."
-                                .format('angle', 'rotate', error_msg))
-
-        # Interpolation parameter sanity
         if order not in range(6):
-            raise ValueError("Order must be between 0 and 5")
+            raise ValueError("Order must be between 0 and 5.")
 
         # The FITS-WCS transform is by definition defined around the
         # reference coordinate in the header.
@@ -1278,17 +1294,17 @@ class GenericMap(NDData):
 
             # Round the pixel values, we use floor+1 so that we always have at
             # least one pixel width of data.
-            x_pixels = u.Quantity([np.min(pixel_corners.x), np.max(pixel_corners.x)]).value
+            x_pixels = u.Quantity([np.min(pixel_corners.x), np.max(pixel_corners.x)]).to_value(u.pix)
             x_pixels[0] = np.ceil(x_pixels[0])
             x_pixels[1] = np.floor(x_pixels[1] + 1)
-            y_pixels = u.Quantity([np.min(pixel_corners.y), np.max(pixel_corners.y)]).value
+            y_pixels = u.Quantity([np.min(pixel_corners.y), np.max(pixel_corners.y)]).to_value(u.pix)
             y_pixels[0] = np.ceil(y_pixels[0])
             y_pixels[1] = np.floor(y_pixels[1] + 1)
 
         elif (isinstance(bottom_left, u.Quantity) and bottom_left.unit.is_equivalent(u.pix) and
               isinstance(top_right, u.Quantity) and top_right.unit.is_equivalent(u.pix)):
-            x_pixels = u.Quantity([bottom_left[0], top_right[0]]).value
-            y_pixels = u.Quantity([top_right[1], bottom_left[1]]).value
+            x_pixels = u.Quantity([bottom_left[0], top_right[0]]).to_value(u.pix)
+            y_pixels = u.Quantity([top_right[1], bottom_left[1]]).to_value(u.pix)
 
         else:
             raise ValueError("Invalid input, bottom_left and top_right must either be SkyCoord or Quantity in pixels.")
@@ -1315,8 +1331,8 @@ class GenericMap(NDData):
 
         # Make a copy of the header with updated centering information
         new_meta = self.meta.copy()
-        new_meta['crpix1'] = self.reference_pixel.x.value - x_pixels[0]
-        new_meta['crpix2'] = self.reference_pixel.y.value - y_pixels[0]
+        new_meta['crpix1'] = self.reference_pixel.x.to_value(u.pix) - x_pixels[0]
+        new_meta['crpix2'] = self.reference_pixel.y.to_value(u.pix) - y_pixels[0]
         new_meta['naxis1'] = new_data.shape[1]
         new_meta['naxis2'] = new_data.shape[0]
 
@@ -1420,6 +1436,19 @@ class GenericMap(NDData):
 
 # #### Visualization #### #
 
+    @property
+    def cmap(self):
+        """
+        Return the `matplotlib.colors.Colormap` instance this map uses.
+        """
+        cmap = self.plot_settings['cmap']
+        if isinstance(cmap, str):
+            cmap = plt.get_cmap(cmap)
+            # Set the colormap to be this specific instance so we are not
+            # returning a copy
+            self.plot_settings['cmap'] = cmap
+        return cmap
+
     @u.quantity_input
     def draw_grid(self, axes=None, grid_spacing: u.deg = 15*u.deg, **kwargs):
         """
@@ -1431,8 +1460,8 @@ class GenericMap(NDData):
 
         Parameters
         ----------
-        axes: `~matplotlib.axes` or None
-        Axes to plot limb on or None to use current axes.
+        axes: `~matplotlib.axes` or `None`
+            Axes to plot limb on, or `None` to use current axes.
 
         grid_spacing: `~astropy.units.Quantity`
             Spacing for longitude and latitude grid, if length two it specifies
@@ -1475,6 +1504,8 @@ class GenericMap(NDData):
         -----
         Keyword arguments are passed onto `matplotlib.patches.Circle`.
         """
+        # Put import here to reduce sunpy.map import time
+        from matplotlib import patches
 
         if not axes:
             axes = wcsaxes_compat.gca_wcs(self.wcs)
